#include "acquisition.h"
#include <timers.h>
#include <math.h>

CurrChannel currentChannel;
unsigned char currGains[3] = {1,10,100};
unsigned char currChannel[3] = {0,1,2};

signed long powSum;
signed long voltSum;
signed long currSum;
unsigned long voltSumRMS;
unsigned long currSumRMS;
int maxCurrent;
int maxVoltage;
int iSample;
int acqReady;
float powAvgSum;
int nbSamplesAvg;
volatile int timeToAcq;

AcqValues acquisition;

void startAcq(){
    //ADC
    ADCON0bits.ADON = 0; //Disable adc

    ADCON1bits.PCFG = 0b1001; // AN0 -> AN4
    ADCON1bits.VCFG0 = 1; // V+ reference
    ADCON1bits.VCFG1 = 0; // Vss reference

    ADCON2bits.ADCS = 0b010; // FOsc/32
    ADCON2bits.ACQT = 0b010; // Acq time

    ADCON2bits.ADFM = 1; // Right justified

    ADCON0bits.ADON = 1; //Enable adc

    currentChannel = CUR_CH_3;
    OpenTimer0(T0_8BIT & T0_SOURCE_INT & T0_PS_1_1);
    INTCONbits.TMR0IE = 1;  // Enable interrupt
    INTCON2bits.TMR0IP = 1; // High priority
    ei();                   // Enable interrupts
    iSample = 0;
    nbSamplesAvg=0;
    powSum=0;
    maxCurrent=0;
    maxVoltage=0;
    acqReady = 0;
    timeToAcq = 0;

    TRISAbits.TRISA7 = 0;
    LATAbits.LATA7=0;
}


void stopAcq(){
    di();
}

void restartAvg(void){
    nbSamplesAvg=0;
    powAvgSum=0;
}

void acqTask(){
    if(timeToAcq){
        timeToAcq=0;
        LATAbits.LATA7=1;
        acqSample();
        LATAbits.LATA7=0;
    }
}

void autoScale(void){
    if(maxCurrent>480){
        switch(currentChannel){
            case CUR_CH_3:
                currentChannel = CUR_CH_2;
                break;
            case CUR_CH_2:
                currentChannel = CUR_CH_1;
                break;
            case CUR_CH_1:
                currentChannel = CUR_CH_1;
            default:
                break;
        }
    }
    else if(maxCurrent<40){
        switch(currentChannel){
            case CUR_CH_1:
                currentChannel = CUR_CH_2;
                break;
            case CUR_CH_2:
                currentChannel = CUR_CH_3;
                break;
            case CUR_CH_3:
                currentChannel = CUR_CH_3;
            default:
                break;
        }
    }
}

void acqSample(){
    // get current
    ADCON0bits.CHS = currChannel[currentChannel]; //Channel 0
    ADCON0bits.GO = 1;
    while (ADCON0bits.NOT_DONE) {
    }
    int current = ADRESH ;
    current <<= 8;
    current += ADRESL;
    current = current-CURRENT_OFFSET1;

    // get voltage
    ADCON0bits.CHS = VOLT_CHAN ; //Channel 4
    ADCON0bits.GO = 1;
    while (ADCON0bits.NOT_DONE) {
    }
    int voltage = ADRESH ;
    voltage <<= 8;
    voltage += ADRESL;
    voltage = voltage-VOLTAGE_OFFSET;

    voltSum+=voltage;
    currSum+=current;
    voltSumRMS+=(long)voltage*(long)(voltage);
    currSumRMS+=(long)current*(long)current;
    powSum += ((long)voltage*(long)current);
    if(current>maxCurrent)
        maxCurrent=current;
    if(voltage>maxVoltage)
        maxVoltage=voltage;
    iSample++;

    // Enough samples
    if(iSample == SAMPLES){
        di();
        acquisition.power = POWER_COEF1*(float)powSum/(float)SAMPLES/currGains[currentChannel];
        acquisition.currentPP = maxCurrent;
        acquisition.voltagePP = maxVoltage;
        acquisition.currentRMS = 1000*sqrt(currSumRMS)*CURR_COEF/currGains[currentChannel];
        acquisition.voltageRMS = sqrt(voltSumRMS)*VOLT_COEF;
        acquisition.powerApparent = acquisition.currentRMS*acquisition.voltageRMS/1000;
        acquisition.gain = currGains[currentChannel];
        if((currentChannel == CUR_CH_1) && maxCurrent > 500){
            acquisition.saturation = 1;
        } else {
            acquisition.saturation = 0;
        }
        powAvgSum += acquisition.power;
        nbSamplesAvg++;
        acquisition.powerAvg = powAvgSum/nbSamplesAvg;

        autoScale();
        voltSum = currSum = voltSumRMS = currSumRMS = 0;
        acqReady = 1;
        iSample=0;
        powSum=0;
        maxCurrent=0;
        maxVoltage=0;
        ei();
    }
}