#include <i2c.h>
#include <string.h>
#include "lcd.h"

void LCDInit(void){
    // I2C LCD
    //SSPCON1bits.SSPEN = 1; // Enable I2C
    //SSPCON1bits.SSPM = 0b1000; //I2C master mode
    I2C_SCL = 1;
    I2C_SDA = 1;

    OpenI2C(MASTER, SLEW_ON);
    SSPADD = 39;
}


void LCDSendI2C(const char* data, int length){
    int i;
    for(i=0;i<length;i++){
        IdleI2C();
        WriteI2C(*(data++));
    }
    IdleI2C();
}

void LCDRecvI2C(char* data, int length){
    RestartI2C();
        IdleI2C();
    WriteI2C(LCD_READ_ADDR);
        IdleI2C();
    int i;
    for(i=0;i<length;i++){
        IdleI2C();
        *(data++) = ReadI2C();
        AckI2C();
            IdleI2C();
    }
    NotAckI2C();
    IdleI2C();
}

void LCDPreSend(){
    StartI2C();
    IdleI2C();
    WriteI2C(LCD_WRITE_ADDR);
        IdleI2C();
}

void LCDPostSend(){
    StopI2C();
}

void LCDSkipAns(void){
    RestartI2C();
        IdleI2C();
    WriteI2C(LCD_READ_ADDR);
        IdleI2C();
    ReadI2C();
        IdleI2C();
    AckI2C();
        IdleI2C();
    ReadI2C();
        IdleI2C();
    NotAckI2C();
        IdleI2C();
}

void LCDSetLed(LedState state){
    char cmd[]={0xB0,0x01,0x25,0x25};
    cmd[1] = state;
    LCDPreSend();
    LCDSendI2C(cmd,4);
    LCDSkipAns();
    LCDPostSend();
}

void LCDWriteLine(LineType line, const unsigned char* str){
    char cmd[]={0x55,0x00,0x00};
    cmd[1] = line;
    LCDPreSend();
    LCDSendI2C(cmd,3);
    WriteI2C(strlen(str));
    IdleI2C();
    LCDSendI2C(str,strlen(str));
    WriteI2C(0x00);
    IdleI2C();
    LCDSkipAns();
    LCDPostSend();
}

void LCDClearLines(){
    char cmd[]={0x88,0x00,0x00};
    LCDPreSend();
    LCDSendI2C(cmd,3);
    LCDSkipAns();
    LCDPostSend();
}

unsigned char LCDReadButtons(void){
    unsigned char resp[3];
    LCDPreSend();
    WriteI2C(0x11);
        IdleI2C();
    LCDRecvI2C(resp,3);
    LCDPostSend();
    return resp[1];
}