/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#if defined(__XC)
#include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
#include <htc.h>        /* HiTech General Include File */
#elif defined(__18CXX)
#include <p18cxxx.h>    /* C18 General Include File */
#endif

#if defined(__XC) || defined(HI_TECH_C)

#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */
#include <i2c.h>
#include <string.h>
#include <stdlib.h>
#include <timers.h>
#include <adc.h>

#endif

#include "user.h"
#include "lcd.h"
#include "acquisition.h"
#include "system.h"

/******************************************************************************/
/* User Functions                                                             */
/******************************************************************************/

/* <Initialize variables in user.h and insert code for user algorithms.> */
void InitApp(void) {
    // IO
    // INTCON2bits.RBPU = 0;
}

void dispValUnit(const char* title, float val, const char* unit) {
    LCDClearLines();
    char titleStr[17] = "                ";
    memcpy(titleStr, title, strlen(title));
    if (acquisition.saturation)
        strcpy(&titleStr[12],"Sat");
    LCDWriteLine(UPPER, titleStr);
    char* valStr;
    int status;
    char valStrfinal[17] = "                ";
    valStr = ftoa(val, &status);
    int pos=4;
    if (valStr[0] == '-'){
       valStr[6] = 0;
       pos=3;
    }
    else
        valStr[5] = 0;
    strcat(valStr, unit);
    strcpy(&valStrfinal[pos],valStr);
    LCDWriteLine(LOWER, valStrfinal);
}

void updateDispMain(void) {
    if (acqReady) {
        di();
        acqReady = 0;
        switch (dispMode) {
            case POW_INST:
                dispValUnit("Puissance inst.", acquisition.power, " W");
                break;
            case POW_APPA:
                dispValUnit("P appa.", acquisition.powerApparent, " W");
                break;
            case POW_AVG:
                dispValUnit("P avg.", acquisition.powerAvg, " W");
                break;
            case VOLT_RMS:
                dispValUnit("U rms", acquisition.voltageRMS, " V");
                break;
            case CURR_RMS:
                dispValUnit("I rms", acquisition.currentRMS, " mA");
                break;
        }
        ei();
    }
}

void checkButtonsMain(void) {
    if (timBtn==0) {
        unsigned char buttons = LCDReadButtons();
        if ((buttons & BTN_UP) == BTN_UP) {
            unsigned char buttons = LCDReadButtons();
            if ((buttons & BTN_UP) == BTN_UP) {
                timBtn = BTN_DELAY;
                if (dispMode == CURR_RMS)
                    dispMode = POW_INST;
                else
                    dispMode++;
                if(dispMode == POW_AVG)
                    restartAvg();
                acqReady=1;
            }
        }
        if ((buttons & BTN_DOWN) == BTN_DOWN) {
            unsigned char buttons = LCDReadButtons();
            if ((buttons & BTN_DOWN) == BTN_DOWN) {
                timBtn = BTN_DELAY;
                if (dispMode == POW_INST)
                    dispMode = CURR_RMS;
                else
                    dispMode--;
                if(dispMode == POW_AVG)
                    restartAvg();
                acqReady=1;
            }
        }
    }
}


void mainScreen(void){
    while (1) {
        acqTask();
        checkButtonsMain();
        updateDispMain();
    }
}


